@@ -46,6 +46,8 @@ Common options:
     -d, --delimiter <arg>      The field delimiter for reading CSV data.
                                Must be a single character. (default: ,)
     -Q, --quiet                Do not print duplicate count to stderr.
+    --no-memcheck              Do not check if there is enough memory to load the
+                               entire CSV into memory.
 "#;
 
 use std::cmp;
@@ -53,6 +55,7 @@ use std::cmp;
 use csv::ByteRecord;
 use rayon::prelude::*;
 use serde::Deserialize;
+use simdutf8::basic::from_utf8;
 
 use crate::{
     cmd::sort::iter_cmp,
@@ -73,6 +76,7 @@ struct Args {
     flag_human_readable: bool,
     flag_jobs:           Option<usize>,
     flag_quiet:          bool,
+    flag_no_memcheck:    bool,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -81,7 +85,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let rconfig = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
         .no_headers(args.flag_no_headers)
-        .checkutf8(false)
         .select(args.flag_select);
 
     let mut rdr = rconfig.reader()?;
@@ -137,9 +140,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     } else {
         // we're loading the entire file into memory, we need to check avail mem
         if let Some(path) = rconfig.path.clone() {
-            util::mem_file_check(&path)?;
+            util::mem_file_check(&path, false, args.flag_no_memcheck)?;
         }
 
+        // set RAYON_NUM_THREADS for parallel sort
         util::njobs(args.flag_jobs);
 
         let mut all = rdr.byte_records().collect::<Result<Vec<_>, _>>()?;
@@ -149,30 +153,31 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             iter_cmp(a, b)
         });
 
-        let mut current = 0;
-        while current + 1 < all.len() {
-            let a = sel.select(&all[current]);
-            let b = sel.select(&all[current + 1]);
-            if ignore_case {
-                if iter_cmp_ignore_case(a, b) == cmp::Ordering::Equal {
+        for (current, current_record) in all.iter().enumerate() {
+            let a = sel.select(current_record);
+            if let Some(next_record) = all.get(current + 1) {
+                let b = sel.select(next_record);
+                if ignore_case {
+                    if iter_cmp_ignore_case(a, b) == cmp::Ordering::Equal {
+                        dupe_count += 1;
+                        if dupes_output {
+                            dupewtr.write_byte_record(current_record)?;
+                        }
+                    } else {
+                        wtr.write_byte_record(current_record)?;
+                    }
+                } else if iter_cmp(a, b) == cmp::Ordering::Equal {
                     dupe_count += 1;
                     if dupes_output {
-                        dupewtr.write_byte_record(&all[current])?;
+                        dupewtr.write_byte_record(current_record)?;
                     }
                 } else {
-                    wtr.write_byte_record(&all[current])?;
-                }
-            } else if iter_cmp(a, b) == cmp::Ordering::Equal {
-                dupe_count += 1;
-                if dupes_output {
-                    dupewtr.write_byte_record(&all[current])?;
+                    wtr.write_byte_record(current_record)?;
                 }
             } else {
-                wtr.write_byte_record(&all[current])?;
+                wtr.write_byte_record(current_record)?;
             }
-            current += 1;
         }
-        wtr.write_byte_record(&all[current])?;
     }
 
     dupewtr.flush()?;
@@ -219,6 +224,6 @@ where
     X: Iterator<Item = &'a [u8]>,
 {
     xs.next()
-        .map(|bytes| unsafe { std::str::from_utf8_unchecked(bytes) })
+        .and_then(|bytes| from_utf8(bytes).ok())
         .map(str::to_lowercase)
 }
