@@ -1,6 +1,6 @@
 static USAGE: &str = r#"
-Create a new computed column, filter rows or compute aggregations by executing a
-Luau script for every row of a CSV file.
+Create multiple new computed columns, filter rows or compute aggregations by 
+executing a Luau script for every row of a CSV file.
 
 The executed Luau has 3 ways to reference row columns (as strings):
   1. Directly by using column name (e.g. Amount), can be disabled with -g
@@ -37,6 +37,9 @@ Some usage examples:
         "if tonumber(Amount) < 0 then return 'debit' else return 'credit' end" | \
     qsv luau map AbsAmount "math.abs(tonumber(Amount))"
 
+  Map multiple new columns in one pass
+  $ qsv luau map newcol1,newcol2,newcol3 "{cola + 1, colb + 2, colc + 3}"
+
   Filter some rows based on numerical filtering
   $ qsv luau filter "tonumber(a) > 45"
   $ qsv luau filter "tonumber(a) >= tonumber(b)"
@@ -45,22 +48,38 @@ Some usage examples:
   "file:" prefix to read non-trivial scripts from the filesystem.
   $ qsv luau map Type -B "file:init.luau" -x "file:debitcredit.luau" -E "file:end.luau"
 
-The MAIN script is evaluated on a per row basis.
-With "luau map", if the main-script is invalid for a row, "<ERROR>" is returned for that row.
-With "luau filter", if the main-script is invalid for a row, that row is not filtered.
+With "luau map", if the MAIN script is invalid for a row, "<ERROR>" is returned for that row.
+With "luau filter", if the MAIN script is invalid for a row, that row is not filtered.
 
 If any row has an invalid result, an exitcode of 1 is returned and an error count is logged.
 
-There are also special variables - "_IDX" - a read-only variable that is zero during the BEGIN script and
-set to the current row number during the main script; & "_ROWCOUNT" - a read-only variable which is zero
-during the BEGIN & MAIN scripts, and set to the rowcount during the END script.
+SPECIAL VARIABLES:
+  "_IDX" - a READ-only variable that is zero during the BEGIN script and
+       set to the current row number during the MAIN & END scripts.
 
-"_INDEX" - a read/write variable that enables random access of the CSV file. Setting it to a row number
-will change the current row to that row number. Setting "_INDEX" to a negative number will start from
-the end of the CSV file. It will only work, however, if the CSV has an index.
+       "_IDX" is primarily used when the CSV has no index and the MAIN script evaluates each
+       row in sequence.
+ 
+  "_INDEX" - a READ/WRITE variable that enables random access of the CSV file. Setting it to 
+       a row number will change the current row to that row number.
+       It will only work, however, if the CSV has an index.
 
-When using _INDEX, the MAIN script will keep looping and evaluate the row specified by _INDEX
-until _INDEX is set to an invalid row number (e.g. negative number or to a value greater than rowcount).
+       When using _INDEX, the MAIN script will keep looping and evaluate the row specified by
+       _INDEX until _INDEX is set to an invalid row number (e.g. negative number or to a value
+       greater than rowcount).
+
+       "_INDEX" is primarily used when you want to process the CSV in random access mode.
+       If the CSV has no index, it will abort with an error unless "qsv_autoindex()" is
+       called in the BEGIN script to create an index.
+       
+  "_ROWCOUNT" - a READ-only variable which is zero during the BEGIN & MAIN scripts, 
+       and set to the rowcount during the END script when the CSV has no index.
+
+       When using _INDEX and the CSV has an index, _ROWCOUNT will be set to the rowcount
+       of the CSV file, even from the BEGINning.
+
+  "_LASTROW" - a READ-only variable that is set to the last row number of the CSV file.
+       It will only work, however, if the CSV has an index.
 
 Luau's standard library is relatively minimal (https://luau-lang.org/library).
 That's why qsv preloads the LuaDate library as date manipulation is a common data-wrangling task.
@@ -72,32 +91,40 @@ See http://lua-users.org/wiki/LibrariesAndBindings for a list of other libraries
 With the judicious use of "require", the BEGIN script & the "_IDX"/"_ROWCOUNT" variables, one can create
 variables/tables/arrays that can be used for complex aggregation operations in the END script.
 
-TIP: When developing Luau scripts, be sure to take advantage of the "qsv_log" function.
-It will log messages to the logfile at the specified log level. Be sure to set QSV_LOG_LEVEL to the
-appropriate log level. The first parameter to qsv_log is the log level (info, warn, error, debug, trace).
+TIP: When developing Luau scripts, be sure to take advantage of the "qsv_log" function to debug your script.
+It will log messages to the logfile at the specified log level as specified by the QSV_LOG_LEVEL
+environment variable. The first parameter to qsv_log is the log level (info, warn, error, debug, trace)
+of the log message and will default to "info" if an invalid log level is specified.
 You can add as many as 255 addl parameters which will be concatenated and logged as a single message.
 
 For more detailed examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_luau.rs.
 
 Usage:
     qsv luau map [options] -n <main-script> [<input>]
-    qsv luau map [options] <new-column> <main-script> [<input>]
+    qsv luau map [options] <new-columns> <main-script> [<input>]
     qsv luau filter [options] <main-script> [<input>]
     qsv luau map --help
     qsv luau filter --help
     qsv luau --help
 
-All <script> arguments/options can either be the Luau code, or if it starts with "file:",
-the filepath from which to load the script.
+Luau arguments:
 
-Instead of using the --begin and --end options, you can also embed BEGIN and END scripts.
-The BEGIN script is embedded in the MAIN script by adding a BEGIN block at the top of the script.
-The BEGIN block is "BEGIN { ... }!" and can contain multiple statements.
+    All <script> arguments/options can either be the Luau code, or if it starts with "file:",
+    the filepath from which to load the script.
 
-The END script is embedded in the MAIN script by adding an END block at the bottom of the script.
-The END block is "END { ... }!" and can contain multiple statements.
+    Instead of using the --begin and --end options, you can also embed BEGIN and END scripts in the
+    MAIN script by using the "BEGIN { ... }!" and "END { ... }!" syntax.
 
-luau options:
+    The BEGIN script is embedded in the MAIN script by adding a BEGIN block at the top of the script.
+    The BEGIN block must start at the beggining of the line. It can contain multiple statements.
+
+    The END script is embedded in the MAIN script by adding an END block at the bottom of the script.
+    The END block must start at the beginning of the line. It can contain multiple statements.
+
+    <new-columns> is a comma-separated list of new computed columns to add to the CSV when using
+    "luau map". Note that the new columns are added to the CSV after the existing columns.
+
+Luau options:
     -x, --exec               exec[ute] Luau script, instead of the default eval[uate].
                              eval (default) expects just a single Luau expression,
                              while exec expects one or more statements, allowing
@@ -106,16 +133,13 @@ luau options:
     -g, --no-globals         Don't create Luau global variables for each column, only col.
                              Useful when some column names mask standard Luau globals.
                              Note: access to Luau globals thru _G remains even without -g.
-    -B, --begin <script>     Luau script/file to execute in the BEGINning, before processing the CSV
-                             with the main-script.
-                             The variables _IDX and _ROWCOUNT are set to zero before invoking
-                             the BEGIN script.
+    -B, --begin <script>     Luau script/file to execute in the BEGINning, before processing
+                             the CSV with the main-script.
                              Typically used to initialize global variables.
                              Takes precedence over an embedded BEGIN script.
-    -E, --end <script>       Luau script/file to execute at the END, after processing the CSV with
-                             the main-script.
-                             Both _IDX and _ROWCOUNT variables are set to the rowcount before invoking
-                             the END script. Typically used for aggregations.
+    -E, --end <script>       Luau script/file to execute at the END, after processing the
+                             CSV with the main-script.
+                             Typically used for aggregations.
                              The output of the END script is sent to stderr.
                              Takes precedence over an embedded END script.
     --luau-path <pattern>    The LUAU_PATH pattern to use from which the scripts 
@@ -130,9 +154,7 @@ Common options:
     -h, --help             Display this message
     -o, --output <file>    Write output to <file> instead of stdout.
     -n, --no-headers       When set, the first row will not be interpreted
-                           as headers. Namely, it will be sorted with the rest
-                           of the rows. Otherwise, the first row will always
-                           appear as the header row in the output.
+                           as headers.
     -d, --delimiter <arg>  The field delimiter for reading CSV data.
                            Must be a single character. (default: ,)
     -p, --progressbar      Show progress bars. Not valid for stdin.
@@ -140,8 +162,9 @@ Common options:
                            for random access.
 "#;
 
-use std::{env, fs};
+use std::{env, fs, io, io::Write, path::Path};
 
+use csv_index::RandomAccessSimple;
 #[cfg(any(feature = "full", feature = "lite"))]
 use indicatif::{ProgressBar, ProgressDrawTarget};
 use log::{debug, info, log_enabled};
@@ -158,7 +181,7 @@ use crate::{
 struct Args {
     cmd_map:          bool,
     cmd_filter:       bool,
-    arg_new_column:   Option<String>,
+    arg_new_columns:  Option<String>,
     arg_main_script:  String,
     arg_input:        Option<String>,
     flag_exec:        bool,
@@ -195,12 +218,18 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         args.arg_main_script.clone()
     };
 
-    let mut index_file_used = luau_script.contains("_INDEX");
+    // in Luau, comments begin with two consecutive hyphens
+    // let's remove them, so we don't falsely trigger on commented special variables
+    let comment_remover_re = regex::Regex::new(r"--.*?$").unwrap();
+    comment_remover_re.replace_all(&luau_script, "");
+
+    let mut index_file_used = luau_script.contains("_INDEX") || luau_script.contains("_LASTROW");
 
     // check if the main script has BEGIN and END blocks
     // and if so, extract them and remove them from the main script
-    let begin_re = regex::Regex::new(r"(?ms)BEGIN \{(?P<begin_block>.*?)\}!").unwrap();
-    let end_re = regex::Regex::new(r"(?ms)END \{(?P<end_block>.*?)\}!").unwrap();
+    let begin_re = regex::Regex::new(r"(?ms)^BEGIN \{(?P<begin_block>.*?)\}!").unwrap();
+    let end_re = regex::Regex::new(r"(?ms)^END \{(?P<end_block>.*?)\}!").unwrap();
+
     let mut embedded_begin_script = String::new();
     let mut embedded_end_script = String::new();
     let mut main_script = luau_script.clone();
@@ -224,6 +253,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     };
 
     main_script.push_str(luau_script.trim());
+    comment_remover_re.replace_all(&main_script, "");
     debug!("MAIN script: {main_script:?}");
 
     // setup LUAU_PATH; create a temporary directory and add it to LUAU_PATH and copy date.lua
@@ -244,11 +274,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let begin_script = if let Some(ref begin) = args.flag_begin {
         if let Some(begin_filepath) = begin.strip_prefix("file:") {
             match fs::read_to_string(begin_filepath) {
-                Ok(begin) => {
-                    // check if the BEGIN script uses _INDEX
-                    index_file_used = index_file_used || begin.contains("_INDEX");
-                    begin
-                }
+                Ok(begin) => begin,
                 Err(e) => return fail_clierror!("Cannot load Luau BEGIN script file: {e}"),
             }
         } else {
@@ -257,17 +283,17 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     } else {
         embedded_begin_script.trim().to_string()
     };
+    comment_remover_re.replace_all(&begin_script, "");
+    // check if the BEGIN script uses _INDEX
+    index_file_used =
+        index_file_used || begin_script.contains("_INDEX") || begin_script.contains("_LASTROW");
     debug!("BEGIN script: {begin_script:?}");
 
     // check if an END script was specified
     let end_script = if let Some(ref end) = args.flag_end {
         if let Some(end_filepath) = end.strip_prefix("file:") {
             match fs::read_to_string(end_filepath) {
-                Ok(end) => {
-                    // check if the END script uses _INDEX
-                    index_file_used = index_file_used || end.contains("_INDEX");
-                    end
-                }
+                Ok(end) => end,
                 Err(e) => return fail_clierror!("Cannot load Luau END script file: {e}"),
             }
         } else {
@@ -276,6 +302,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     } else {
         embedded_end_script.trim().to_string()
     };
+    comment_remover_re.replace_all(&end_script, "");
+    // check if the END script uses _INDEX
+    index_file_used =
+        index_file_used || end_script.contains("_INDEX") || end_script.contains("_LASTROW");
     debug!("END script: {end_script:?}");
 
     // -------- setup Luau environment --------
@@ -294,10 +324,43 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             .set_debug_level(1)
             .set_coverage_level(0)
     };
+    // set default Luau compiler
     luau.set_compiler(luau_compiler.clone());
 
     let globals = luau.globals();
 
+    setup_helpers(&luau)?;
+
+    if index_file_used {
+        with_index(
+            &rconfig,
+            &args,
+            &luau,
+            &luau_compiler,
+            &globals,
+            &begin_script,
+            &main_script,
+            &end_script,
+            args.flag_max_errors,
+        )?;
+    } else {
+        no_index(
+            &rconfig,
+            &args,
+            &luau,
+            &luau_compiler,
+            &globals,
+            &begin_script,
+            &main_script,
+            &end_script,
+            args.flag_max_errors,
+        )?;
+    }
+
+    Ok(())
+}
+
+fn setup_helpers(luau: &Lua) -> Result<(), CliError> {
     // this is a helper function that can be called from Luau scripts
     // to send log messages to the logfile
     // the first parameter is the log level, and the following parameters are concatenated
@@ -332,32 +395,26 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     })?;
     luau.globals().set("qsv_log", qsv_log)?;
 
-    if index_file_used {
-        with_index(
-            &rconfig,
-            &args,
-            &luau,
-            &luau_compiler,
-            &globals,
-            &begin_script,
-            &main_script,
-            &end_script,
-            args.flag_max_errors,
-        )?;
-    } else {
-        no_index(
-            &rconfig,
-            &args,
-            &luau,
-            &luau_compiler,
-            &globals,
-            &begin_script,
-            &main_script,
-            &end_script,
-            args.flag_max_errors,
-        )?;
-    }
+    // this is a helper function that can be called from Luau scripts
+    // to coalesce - return the first non-null value in a list
+    let qsv_coalesce = luau.create_function(|luau, mut args: mlua::MultiValue| {
+        while let Some(val) = args.pop_front() {
+            let val = luau.from_value::<serde_json::Value>(val)?;
+            let val_str = &serde_json::to_string(&val).unwrap_or_default();
+            if !val_str.is_empty() {
+                return Ok(val_str.to_string());
+            }
+        }
+        Ok(String::new())
+    })?;
+    luau.globals().set("qsv_coalesce", qsv_coalesce)?;
 
+    // this is a helper function that creates an index file for the current CSV.
+    // It does not work for stdin and should only be called in the BEGIN script
+    // its actually just a stub and the real function is called before processing
+    // the BEGIN script
+    let qsv_autoindex = luau.create_function(|_luau, mut _args: mlua::MultiValue| Ok(true))?;
+    luau.globals().set("qsv_autoindex", qsv_autoindex)?;
     Ok(())
 }
 
@@ -392,13 +449,19 @@ fn no_index(
     let mut rdr = rconfig.reader()?;
     let mut wtr = Config::new(&args.flag_output).writer()?;
     let mut headers = rdr.headers()?.clone();
+    let mut new_column_count = 0_u8;
     if !rconfig.no_headers {
         if !args.cmd_filter {
-            let new_column = args
-                .arg_new_column
+            let new_columns = args
+                .arg_new_columns
                 .as_ref()
-                .ok_or("Specify new column name")?;
-            headers.push_field(new_column);
+                .ok_or("Specify new column names")?;
+
+            let new_columns_vec: Vec<&str> = new_columns.split(',').collect();
+            for new_column in new_columns_vec {
+                new_column_count += 1;
+                headers.push_field(new_column.trim());
+            }
         }
 
         wtr.write_record(&headers)?;
@@ -494,6 +557,49 @@ fn no_index(
                 Value::Nil => {
                     record.push_field("");
                 }
+                Value::Table(table) => {
+                    let mut columns_inserted = 0_u8;
+                    for pair in table.pairs::<mlua::Value, mlua::Value>() {
+                        // we don't care about the key, just the value
+                        let (_k, v) = pair?;
+                        match v {
+                            Value::Integer(intval) => {
+                                let mut buffer = itoa::Buffer::new();
+                                record.push_field(buffer.format(intval));
+                            }
+                            Value::String(strval) => {
+                                record.push_field(&strval.to_string_lossy());
+                            }
+                            Value::Number(number) => {
+                                let mut buffer = ryu::Buffer::new();
+                                record.push_field(buffer.format(number));
+                            }
+                            Value::Boolean(boolean) => {
+                                record.push_field(if boolean { "true" } else { "false" });
+                            }
+                            Value::Nil => {
+                                record.push_field("");
+                            }
+                            _ => {
+                                return fail_clierror!(
+                                    "Unexpected value type returned by provided Luau expression. \
+                                     {v:?}"
+                                );
+                            }
+                        }
+                        columns_inserted += 1;
+                        if new_column_count > 0 && columns_inserted >= new_column_count {
+                            // we ignore table values more than the number of new columns defined
+                            break;
+                        }
+                    }
+                    // on the other hand, if there are less table values than expected
+                    // we fill it up with empty fields
+                    while new_column_count > 0 && columns_inserted < new_column_count {
+                        record.push_field("");
+                        columns_inserted += 1;
+                    }
+                }
                 _ => {
                     return fail_clierror!(
                         "Unexpected value type returned by provided Luau expression. \
@@ -573,8 +679,34 @@ fn no_index(
     Ok(())
 }
 
-// this function is largely similar to no_index
-// the primary difference being that we use an Indexed File rdr in the main loop
+fn create_index(arg_input: Option<String>) -> Result<bool, CliError> {
+    // this is a utility function that creates an index file for the current CSV.
+    // it is called when "qsv_autoindex()" is called in the BEGIN script.
+    let Some(input ) = &arg_input else {
+        log::warn!("qsv_autoindex() does not work for stdin.");
+        return Ok(false);
+    };
+
+    let pidx = util::idx_path(Path::new(&input));
+    debug!("Creating index file {pidx:?} for {input:?}.");
+
+    let rconfig = Config::new(&Some(input.to_string()));
+    let mut rdr = rconfig.reader_file()?;
+    let mut wtr = io::BufWriter::new(fs::File::create(pidx)?);
+    if RandomAccessSimple::create(&mut rdr, &mut wtr).is_err() {
+        return Ok(false);
+    };
+    if wtr.flush().is_err() {
+        return Ok(false);
+    }
+
+    log::info!("qsv_autoindex() successful.");
+    Ok(true)
+}
+
+// this function is largely similar to no_index, and is triggered when
+// we use the special variable _INDEX in the Luau scripts.
+// the primary difference being that we use an Indexed File rdr in the main loop.
 // differences pointed out in comments below
 fn with_index(
     rconfig: &Config,
@@ -587,32 +719,51 @@ fn with_index(
     end_script: &str,
     max_errors: usize,
 ) -> Result<(), CliError> {
-    let Some(mut idx_file) = rconfig.indexed()? else { return fail!("Index required but no index file found") };
+    // qsv_autoindex() was called in the BEGIN script, so we need to create the index file
+    if begin_script.contains("qsv_autoindex()") {
+        let result = create_index(args.arg_input.clone());
+        if result.is_err() {
+            return fail_clierror!("Unable to create index file");
+        }
+    }
+
+    let Some(mut idx_file) = rconfig.indexed()? else {
+        return fail!(r#"Index required but no index file found. Use "qsv_autoindex()" in your BEGIN script."#); 
+    };
 
     globals.set("cols", "{}")?;
 
-    let row_count = util::count_rows(rconfig).unwrap_or_default();
+    let mut row_count = util::count_rows(rconfig).unwrap_or_default();
+    if args.flag_no_headers {
+        row_count += 1;
+    }
 
     let mut wtr = Config::new(&args.flag_output).writer()?;
     let mut headers = idx_file.headers()?.clone();
+    let mut new_column_count = 0_u8;
     if !rconfig.no_headers {
         if !args.cmd_filter {
-            let new_column = args
-                .arg_new_column
+            let new_columns = args
+                .arg_new_columns
                 .as_ref()
-                .ok_or("Specify new column name")?;
-            headers.push_field(new_column);
+                .ok_or("Specify new column names")?;
+
+            let new_columns_vec: Vec<&str> = new_columns.split(',').collect();
+            for new_column in new_columns_vec {
+                new_column_count += 1;
+                headers.push_field(new_column.trim());
+            }
         }
 
         wtr.write_record(&headers)?;
     }
 
+    // unlike no_index, we actually know the row_count at the BEGINning
+    globals.set("_IDX", 0)?;
+    globals.set("_INDEX", 0)?;
+    globals.set("_ROWCOUNT", row_count)?;
+    globals.set("_LASTROW", row_count - 1)?;
     if !begin_script.is_empty() {
-        // unlike no_index, we actually know the row_count at the BEGINning
-        globals.set("_IDX", 0)?;
-        globals.set("_INDEX", 0)?;
-        globals.set("_ROWCOUNT", row_count)?;
-
         info!("Compiling and executing BEGIN script.");
         let begin_bytecode = luau_compiler.compile(begin_script);
         if let Err(e) = luau
@@ -630,8 +781,6 @@ fn with_index(
     let mut pos = globals.get::<_, isize>("_INDEX").unwrap_or_default();
     let mut curr_record = if pos > 0 && pos <= row_count as isize {
         pos as u64
-    } else if pos < 0 && pos.unsigned_abs() as u64 <= row_count {
-        row_count - pos.unsigned_abs() as u64
     } else {
         0_u64
     };
@@ -710,6 +859,49 @@ fn with_index(
                 Value::Nil => {
                     record.push_field("");
                 }
+                Value::Table(table) => {
+                    let mut columns_inserted = 0_u8;
+                    for pair in table.pairs::<mlua::Value, mlua::Value>() {
+                        // we don't care about the key, just the value
+                        let (_k, v) = pair?;
+                        match v {
+                            Value::Integer(intval) => {
+                                let mut buffer = itoa::Buffer::new();
+                                record.push_field(buffer.format(intval));
+                            }
+                            Value::String(strval) => {
+                                record.push_field(&strval.to_string_lossy());
+                            }
+                            Value::Number(number) => {
+                                let mut buffer = ryu::Buffer::new();
+                                record.push_field(buffer.format(number));
+                            }
+                            Value::Boolean(boolean) => {
+                                record.push_field(if boolean { "true" } else { "false" });
+                            }
+                            Value::Nil => {
+                                record.push_field("");
+                            }
+                            _ => {
+                                return fail_clierror!(
+                                    "Unexpected value type returned by provided Luau expression. \
+                                     {v:?}"
+                                );
+                            }
+                        }
+                        columns_inserted += 1;
+                        if new_column_count > 0 && columns_inserted >= new_column_count {
+                            // we ignore table values more than the number of new columns defined
+                            break;
+                        }
+                    }
+                    // on the other hand, if there are less table values than expected
+                    // we fill it up with empty fields
+                    while new_column_count > 0 && columns_inserted < new_column_count {
+                        record.push_field("");
+                        columns_inserted += 1;
+                    }
+                }
                 _ => {
                     return fail_clierror!(
                         "Unexpected value type returned by provided Luau expression. \
