@@ -103,6 +103,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         let stdin = std::io::stdin();
         let mut stdin_handle = stdin.lock();
         std::io::copy(&mut stdin_handle, &mut stdin_file)?;
+        drop(stdin_handle);
         args.arg_input = Some(STDIN_CSV.to_string());
         (STDIN_CSV.to_string(), STDIN_CSV.to_string())
     } else {
@@ -407,7 +408,8 @@ fn get_stats_records(args: &Args) -> CliResult<(ByteRecord, Vec<Stats>, AHashMap
         Ok(o) => {
             if let Some(idx) = o {
                 info!("has index, triggering parallel stats");
-                stats_args.parallel_stats(&stats_args.flag_dates_whitelist, &idx)
+                let idx_count = idx.count();
+                stats_args.parallel_stats(&stats_args.flag_dates_whitelist, idx_count)
             } else {
                 info!("no index, triggering sequential stats");
                 stats_args.sequential_stats(&stats_args.flag_dates_whitelist)
