@@ -38,6 +38,8 @@ Common options:
                             appear as the header row in the output.
     -d, --delimiter <arg>   The field delimiter for reading CSV data.
                             Must be a single character. (default: ,)
+    --no-memcheck           Do not check if there is enough memory to load the
+                            entire CSV into memory.
 "#;
 
 use std::cmp;
@@ -45,6 +47,7 @@ use std::cmp;
 use rand::{rngs::StdRng, seq::SliceRandom, SeedableRng};
 use rayon::prelude::*;
 use serde::Deserialize;
+use simdutf8::basic::from_utf8;
 
 use self::Number::{Float, Int};
 use crate::{
@@ -68,6 +71,7 @@ struct Args {
     flag_no_headers:  bool,
     flag_delimiter:   Option<Delimiter>,
     flag_unique:      bool,
+    flag_no_memcheck: bool,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -78,12 +82,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let rconfig = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
         .no_headers(args.flag_no_headers)
-        .checkutf8(false)
         .select(args.flag_select);
 
     // we're loading the entire file into memory, we need to check avail mem
     if let Some(path) = rconfig.path.clone() {
-        util::mem_file_check(&path)?;
+        util::mem_file_check(&path, false, args.flag_no_memcheck)?;
     }
 
     let mut rdr = rconfig.reader()?;
@@ -237,7 +240,7 @@ where
     X: Iterator<Item = &'a [u8]>,
 {
     xs.next()
-        .map(|bytes| unsafe { std::str::from_utf8_unchecked(bytes) })
+        .map(|bytes| from_utf8(bytes).unwrap())
         .and_then(|s| {
             if let Ok(i) = s.parse::<i64>() {
                 Some(Number::Int(i))
