@@ -133,7 +133,7 @@ It underpins the `schema` and `validate` commands - enabling the automatic creat
 a JSONschema based on a CSV's summary statistics; and use the generated JSONschema to
 quickly validate complex CSVs (NYC's 311 data) at almost 300,000 records/sec.
 
-These "unsafe" calls primarily skip repetitive UTF-8 validation and unneeded bounds checking.
+These "unsafe" calls primarily skip unneeded bounds checking.
 
 To safeguard against undefined behavior, `stats` is the most extensively tested command,
 with ~470 tests.
@@ -142,7 +142,7 @@ with ~470 tests.
 use std::{
     borrow::ToOwned,
     default::Default,
-    fmt, fs, io,
+    fmt, io,
     iter::repeat,
     str::{self, FromStr},
     sync::atomic::{AtomicBool, Ordering},
@@ -159,7 +159,6 @@ use threadpool::ThreadPool;
 use self::FieldType::{TDate, TDateTime, TFloat, TInteger, TNull, TString};
 use crate::{
     config::{Config, Delimiter},
-    index::Indexed,
     select::{SelectColumns, Selection},
     util, CliResult,
 };
@@ -230,14 +229,15 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let (headers, stats) = match fconfig.indexed()? {
         None => args.sequential_stats(&args.flag_dates_whitelist),
         Some(idx) => {
+            let idx_count = idx.count();
             if let Some(num_jobs) = args.flag_jobs {
                 if num_jobs == 1 {
                     args.sequential_stats(&args.flag_dates_whitelist)
                 } else {
-                    args.parallel_stats(&args.flag_dates_whitelist, &idx)
+                    args.parallel_stats(&args.flag_dates_whitelist, idx_count)
                 }
             } else {
-                args.parallel_stats(&args.flag_dates_whitelist, &idx)
+                args.parallel_stats(&args.flag_dates_whitelist, idx_count)
             }
         }
     }?;
@@ -277,11 +277,11 @@ impl Args {
     pub fn parallel_stats(
         &self,
         whitelist: &str,
-        idx: &Indexed<fs::File, fs::File>,
+        idx_count: u64,
     ) -> CliResult<(csv::ByteRecord, Vec<Stats>)> {
         // N.B. This method doesn't handle the case when the number of records
         // is zero correctly. So we use `sequential_stats` instead.
-        if idx.count() == 0 {
+        if idx_count == 0 {
             return self.sequential_stats(whitelist);
         }
 
@@ -295,23 +295,30 @@ impl Args {
             whitelist,
         )?;
 
-        let chunk_size = util::chunk_size(idx.count() as usize, util::njobs(self.flag_jobs));
-        let nchunks = util::num_of_chunks(idx.count() as usize, chunk_size);
+        let chunk_size = util::chunk_size(idx_count as usize, util::njobs(self.flag_jobs));
+        let nchunks = util::num_of_chunks(idx_count as usize, chunk_size);
 
         let pool = ThreadPool::new(util::njobs(self.flag_jobs));
         let (send, recv) = channel::bounded(0);
         for i in 0..nchunks {
-            let (send, args, sel) = (send.clone(), self.clone(), sel.clone());
-            pool.execute(move || unsafe {
-                let mut idx = args
-                    .rconfig()
-                    .indexed()
-                    .unwrap_unchecked()
-                    .unwrap_unchecked();
-                idx.seek((i * chunk_size) as u64).unwrap_unchecked();
-                let it = idx.byte_records().take(chunk_size);
-                send.send(args.compute(&sel, it)).unwrap_unchecked();
-            });
+            // safety: the index file call is always safe
+            // seeking may fail and you have a bigger problem
+            // as the index file was modified while stats is running
+            // so we need to abort if that happens
+            unsafe {
+                let (send, args, sel) = (send.clone(), self.clone(), sel.clone());
+                pool.execute(move || {
+                    let mut idx = args
+                        .rconfig()
+                        .indexed()
+                        .unwrap_unchecked()
+                        .unwrap_unchecked();
+                    idx.seek((i * chunk_size) as u64)
+                        .expect("File seek failed.");
+                    let it = idx.byte_records().take(chunk_size);
+                    send.send(args.compute(&sel, it)).unwrap_unchecked();
+                });
+            }
         }
         drop(send);
         Ok((headers, merge_all(recv.iter()).unwrap_or_default()))
@@ -343,7 +350,7 @@ impl Args {
     {
         let mut stats = self.new_stats(sel.len());
 
-        // we use unsafe here so we skip unnecessary bounds checking
+        // safety: we use unsafe in this hot loop so we skip unnecessary bounds checking
         // that we are certain is safe
         unsafe {
             for row in it {
@@ -969,7 +976,7 @@ impl Stats {
 
                         // and truncate at 100 characters with an ellipsis
                         if antimodes_list.len() > 100 {
-                            antimodes_list.truncate(100);
+                            util::utf8_truncate(&mut antimodes_list, 101);
                             antimodes_list.push_str("...");
                         }
 
@@ -1218,7 +1225,7 @@ impl TypedMinMax {
             return;
         }
         self.strings.add(sample.to_vec());
-        // we can use unwrap_unchecked below since we know the data type of the sample
+        // safety: we can use unwrap_unchecked below since we know the data type of the sample
         unsafe {
             match typ {
                 TString | TNull => {}
