@@ -123,6 +123,8 @@ pub fn version() -> String {
     }
     #[cfg(all(feature = "to", not(feature = "lite")))]
     enabled_features.push_str("to;");
+    #[cfg(all(feature = "polars", not(feature = "lite")))]
+    enabled_features.push_str("polars;");
     #[cfg(feature = "self_update")]
     enabled_features.push_str("self_update");
     enabled_features.push('-');
@@ -216,7 +218,7 @@ pub fn count_rows(conf: &Config) -> Result<u64, CliError> {
         // that don't have an index.
         let rc = ROW_COUNT.get_or_init(|| {
             if let Ok(mut rdr) = conf.reader() {
-                let mut count = 0u64;
+                let mut count = 0_u64;
                 let mut record = csv::ByteRecord::new();
                 while rdr.read_byte_record(&mut record).unwrap_or_default() {
                     count += 1;
@@ -279,8 +281,8 @@ macro_rules! update_cache_info {
             Ok(cache) => {
                 let cache_size = cache.cache_size();
                 if cache_size > 0 {
-                    let hits = cache.cache_hits().expect("Cache hits required");
-                    let misses = cache.cache_misses().expect("Cache misses required");
+                    let hits = cache.cache_hits().unwrap_or_default();
+                    let misses = cache.cache_misses().unwrap_or(1);
                     let hit_ratio = (hits as f64 / (hits + misses) as f64) * 100.0;
                     $progress.set_message(format!(
                         " of {} records. Cache hit ratio: {hit_ratio:.2}%",
@@ -924,7 +926,7 @@ pub fn is_safe_name(header_name: &str) -> bool {
 pub fn log_end(mut qsv_args: String, now: std::time::Instant) {
     if log::log_enabled!(log::Level::Info) {
         let ellipsis = if qsv_args.len() > 24 {
-            qsv_args.truncate(24);
+            utf8_truncate(&mut qsv_args, 24);
             "..."
         } else {
             ""
@@ -936,6 +938,23 @@ pub fn log_end(mut qsv_args: String, now: std::time::Instant) {
     }
 }
 
+// taken from https://gist.github.com/dginev/f6da5e94335d545e0a7b
+pub fn utf8_truncate(input: &mut String, maxsize: usize) {
+    let mut utf8_maxsize = input.len();
+    if utf8_maxsize >= maxsize {
+        {
+            let mut char_iter = input.char_indices();
+            while utf8_maxsize >= maxsize {
+                utf8_maxsize = match char_iter.next_back() {
+                    Some((index, _)) => index,
+                    _ => 0,
+                };
+            }
+        } // Extra {} wrap to limit the immutable borrow of char_indices()
+        input.truncate(utf8_maxsize);
+    }
+}
+
 #[test]
 #[cfg(feature = "self_update")]
 fn test_hw_survey() {
@@ -1034,9 +1053,7 @@ pub fn round_num(dec_f64: f64, places: u32) -> String {
     // use from_f64_retain, so we have all the excess bits before rounding with
     // round_dp_with_strategy as from_f64 will prematurely round when it drops the excess bits
     let Some(dec_num) = Decimal::from_f64_retain(dec_f64) else {
-        let msg = format!(r#"Failed to convert to decimal "{dec_f64}""#);
-        log::error!("{msg}");
-        return msg;
+        return String::new();
     };
 
     // round using Midpoint Nearest Even Rounding Strategy AKA "Bankers Rounding."
